#!/usr/bin/perl
#
#
# SYNTAX:
# 	upgradedb_reports.pl [-t]
#
# DESCRIPTION:
# 	Connects to DB and upgrades it to the latest level
#	-t means running on a test system
#
#	Warning!!!! This file must be kept up to date with db_reports
#
#	Warning #2 !!! Only use DBI commands - no Class::DBI allowed
#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

use strict;
use Getopt::Std;
use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc";
use Reports;

# Do not use Class::DBI methods to amend data
use Utils::DBVersion;

my $opts = {};
getopts( "t", $opts ) or die "Bad options";

my $dbh = Reports->db_Main;

my $db = Utils::DBVersion->new( { dbh => $dbh, name => "reports" } );

# Set no stdout buffering, needed due to top level tee
$| = 1;

if ( $db->is_lower("2.12.1") ) {
    $dbh->do("ALTER TABLE report_types ADD COLUMN invoker varchar(128) AFTER fixedname");
    $dbh->do("ALTER TABLE report_types ADD COLUMN frequency varchar(128) AFTER invoker");
    $dbh->do("ALTER TABLE report_types ADD COLUMN day_value INT AFTER frequency");
    $dbh->do("ALTER TABLE report_types ADD COLUMN requires_selection INT AFTER day_value");
    $dbh->do(
        'CREATE TABLE weekly_availability_keyword_critical_states (
		reportid int NOT NULL,
		start_of_week SMALLINT NOT NULL DEFAULT 1,
		label ENUM ("hostname/servicecheck", "servicecheck", "description") DEFAULT "hostname/servicecheck" NOT NULL,
		keywords TEXT,
		annotation TEXT,
		auto_null_annotation int DEFAULT 0,
		PRIMARY KEY (reportid),
		CONSTRAINT weekly_availability_keyword_critical_states FOREIGN KEY (reportid) REFERENCES reports(id)
	) ENGINE=InnoDB'
    );
    $dbh->do("UPDATE report_types SET requires_selection=1,invoker='generic_availability',frequency='monthly',day_value=1 WHERE id=1");
    $db->updated;
}

if ( $db->is_lower("3.5.1") ) {
    if ( !$dbh->selectrow_array("SELECT COUNT(*) FROM report_types WHERE id=2") ) {
        $dbh->do("INSERT INTO report_types VALUES (2, 'Weekly Availability by Keyword, Critical States', 'weekly_availability_keyword_critical_states', 'generic_availability', 'weekly', 1, 0)");
    }
    $db->updated;
}

if ( $db->changed ) {
    print "Finished updating Reports database", $/;
}
else {
    print "Reports database already up to date", $/;
}
