#
#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

package Opsview::Snmptrap;
use base 'Opsview';

use strict;
our $VERSION = '$Revision: 1172 $';

__PACKAGE__->table("snmptraps");

__PACKAGE__->columns( All => qw/id name allow message alertlevel uncommitted/ );

__PACKAGE__->has_many( actions => [ "Opsview::ServicecheckSnmpaction" => 'servicecheckid' ], "trapid" );
__PACKAGE__->has_many( ignores => [ "Opsview::ServicecheckSnmpignore" => 'servicecheckid' ], "trapid" );

__PACKAGE__->initial_columns(qw/name/);

__PACKAGE__->default_search_attributes( { order_by => "name" } );

sub alertlevel_text {
    my $self = shift;
    if ( ( $_ = $self->alertlevel ) == 0 ) {
        return "OK";
    }
    elsif ( $_ == 1 ) {
        return "WARNING";
    }
    else {
        return "CRITICAL";
    }
}

=head1 NAME

Opsview::Snmptrap - Accessing snmptraps table

=head1 SYNOPSIS

TODO

=head1 DESCRIPTION

Handles interaction with database for Opsview SNMP trap information

=head1 METHODS

=over 4

=item $class->retrieve_allowed_traps

Returns an Class::DBI::Iterator of traps that are allowed for processing

=cut

sub retrieve_allowed_traps {
    my $class = shift;
    my $it    = $class->retrieve_from_sql(
        qq{
                allow = 1
                ORDER BY name
        }
    );
    return $it;
}

=back

=head1 AUTHOR

Opsera Limited

=head1 LICENSE

GNU General Public License v2

=cut

1;
