#
#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

package Opsview::Snmptrapdebug;
use base qw/Runtime/;

use strict;
use DateTime;

our $VERSION = '$Revision: 1279 $';

__PACKAGE__->table("snmptrapdebug");

__PACKAGE__->columns( All => qw/id starttime duration packet/ );

__PACKAGE__->has_a(
    starttime => 'DateTime',
    inflate   => sub { DateTime->from_epoch( epoch => shift, time_zone => "local" ) },
    deflate => sub { shift->epoch }
);

__PACKAGE__->has_many( rules => [ "Opsview::Snmptrapruledebug" => "trap" ] );

__PACKAGE__->default_search_attributes( { order_by => "id DESC" } );

=head1 NAME

Opsview::Snmptrapdebug - Accessing snmptrapdebug table

=head1 SYNOPSIS

TODO

=head1 DESCRIPTION

Handles interaction with database for Opsview SNMP debug tables

=head1 METHODS

=over 4

=item count_all_rules

Not sure why we need this as I'm sure there's a better way

=cut

__PACKAGE__->set_sql(
    count_all_rules => qq{
        SELECT COUNT(*)
        FROM snmptrapruledebug
        WHERE snmptrapruledebug.trap = %s
        }
);

sub count_all_rules {
    my $self = shift;
    $self->sql_count_all_rules( $self->id )->select_val;
}

=item display_packet

Returns the packet, but with the community removed. Duplicated from snmptrapexception

=cut

sub display_packet {
    my $self = shift;
    if ( $_ = $self->packet ) {
        s/\nSNMP-COMMUNITY-MIB::snmpTrapCommunity.0 "(.*?)"/\nSNMP-COMMUNITY-MIB::snmpTrapCommunity.0 "*****"/;
    }
    return $_;
}

=back

=head1 AUTHOR

Opsera Limited

=head1 LICENSE

GNU General Public License v2

=cut

1;
