#
#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

package Opsview::Snmptrapexception;
use base 'Runtime';

use strict;
our $VERSION = '$Revision: 2518 $';

__PACKAGE__->table("snmptrapexceptions");

__PACKAGE__->columns( All  => qw/id time packet hostip reason arg1 trapname trapdebug hostname/ );
__PACKAGE__->columns( TEMP => qw/similar_count/ );

# Do not add this in at the moment as this will load other model files
# When performance of snmptrap2nagios is resolved, can re-add back in
#__PACKAGE__->has_a( trapdebug => "Opsview::Snmptrapdebug" );

__PACKAGE__->default_search_attributes( { order_by => "time DESC" } );

__PACKAGE__->set_sql( delete_debug => "UPDATE __TABLE__ SET trapdebug=NULL" );

=head1 NAME

Opsview::Snmptrapexceptions - Accessing snmptrapexceptions table

=head1 SYNOPSIS

TODO

=head1 DESCRIPTION

Handles interaction with database for Opsview SNMP trap exceptions

=head1 METHODS

=over 4

=item $self->reason_text

Returns the reason text

=cut

my $reason_codes = {
    1 => "No trapname",
    2 => "Trapname not fully translated",
    3 => "Unrecognised trapname",                            # Deprecated as part of basic snmptraps
    4 => "Not expecting trap for this host IP",              # Deprecated as part of basic snmptraps
    5 => "Not monitoring this host IP",
    6 => 'At least 1 servicecheck failed to match a rule',
    7 => 'Requires re-processing',
};

sub reason_text {
    my $self = shift;
    my $text = $reason_codes->{ $self->reason };
    return $text;
}

=item Class->search_unique

Returns an iterator of unique traps, grouped by hostip, trapname, reason

=cut

# This only works because id increases with time
# However, because of multiple slaves, id_of_X < id_of_Y does not imply time_of_X < time_of_Y
# unless it is the same slave
# So you should order by time as well - this is done in controller
# Make sure indexes are created well
__PACKAGE__->set_sql(
    unique => qq{
	SELECT MAX(id) AS id, COUNT(*) AS similar_count
	FROM %s
	WHERE %s
	GROUP BY hostip, trapname, reason
	%s %s
	}
);

=item $self->delete_all_similar

Deletes all traps with similar to $self

=cut

sub delete_all_similar {
    my $self = shift;
    my $sth  = $self->db_Main->prepare_cached( "DELETE FROM " . $self->table . " WHERE hostip = ? AND trapname = ? AND reason = ?" );
    $sth->execute( $self->hostip, $self->trapname, $self->reason );
}

=item display_packet

Returns the packet, but with the community removed

=cut

sub display_packet {
    my $self = shift;
    if ( $_ = $self->packet ) {

        # Why is \n there?
        s/\nSNMP-COMMUNITY-MIB::snmpTrapCommunity.0 "(.*?)"/\nSNMP-COMMUNITY-MIB::snmpTrapCommunity.0 "*****"/;
    }
    return $_;
}

=back

=head1 AUTHOR

Opsera Limited

=head1 LICENSE

GNU General Public License v2

=cut

1;
