#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
package Opsview::Utils::NDOLogsImporter;

sub new {
    my ( $class, $attrs ) = @_;
    my $rootdir = "/usr/local/nagios";
    my $obj     = {
        file2sock  => "$rootdir/bin/file2sock",
        ndo_socket => "$rootdir/var/ndo.sock",
        %$attrs,
    };
    bless $obj, $class;
}

sub last_file_size { shift->{last_file_size} }

sub send_log {
    my ( $self, $file ) = @_;
    my $file_size = $self->{last_file_size} = -s $file;
    if ($file_size) {
        my $c     = 0;
        my $break = 0;

        # $self->{break} is the global break
        until ( $break || ${ $self->{break} } ) {
            $self->{logger}->debug("Importing $file. Size=$file_size");
            my @cmd    = ( $self->{file2sock}, "-s", $file, "-d", $self->{ndo_socket}, "-t", "unix" );
            my @output = `@cmd 2>&1`;
            my $rc     = $? >> 8;
            if ( $rc == 0 ) {
                $break++;
            }
            else {
                $self->{logger}->error("file2sock command '@cmd' failed: rc=$rc, output=@output");
                if ( $output[0] =~ /Connection refused/ ) {

                    # Wait a bit then retry
                    sleep 1;
                    $c++;
                    $self->{logger}->warn("Retrying attempt $c");
                }
                else {
                    $break++;
                }
            }
        }
    }
    else {
        $self->{logger}->info("Ignore $file as size 0");
    }
}

1;
