#
#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

package Reports::GenericAvailabilityTargetsHostgroup;
use warnings;
use strict;
use base "Reports";

__PACKAGE__->table("generic_availability_targets_hostgroup");

__PACKAGE__->columns( All => qw/id hostgroup warning critical/, );

# Has to match Opsview::Hostgroup->name
__PACKAGE__->constrain_column_regexp( hostgroup => q{/(^[\w .\/\+-]+$|^$)/} => q{Hostgroup name can only consist of alphanumerics, space or the following characters: -./+} );

__PACKAGE__->constrain_column_regexp( warning  => '/(^\d*(\.\d+)?$|^$|^default$)/' => "Must be a number" );
__PACKAGE__->constrain_column_regexp( critical => '/(^\d*(\.\d+)?$|^$|^default$)/' => "Must be a number" );

__PACKAGE__->add_constraint( 'only 1 null name', hostgroup => \&check_only_one_null_hostgroup_name );

sub check_only_one_null_hostgroup_name {
    my ( $value, $self ) = @_;
    my $c = $self->search( hostgroup => undef )->count;
    if ( $c > 1 ) {
        return 0;
    }
    return 1;
}

=head1 NAME

Reports::GenericAvailabilityTargetHostgroups - Sets hostgroup warning and critical target levels for this specific report

=head1 METHODS

=over 4

=back

=head1 AUTHOR

Opsera Limited

=head1 LICENSE

GNU General Public License v2

=cut

1;
