#
#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

package Reports::Selection;
use warnings;
use strict;
use base "Reports";

__PACKAGE__->table("selections");

__PACKAGE__->columns( All => qw/id name/, );

__PACKAGE__->initial_columns(qw(name));

__PACKAGE__->constrain_column_regexp( name => q{/^[\w .\/\+-]+$/} => q{Hostgroup name can only consist of alphanumerics, space or the following characters: -./+} );

__PACKAGE__->default_search_attributes( { order_by => "name" } );

__PACKAGE__->has_many( reports => "Reports::Report", { cascade => "Fail" } );
__PACKAGE__->has_many( hostgroups   => "Reports::SelectionHostgroup" );
__PACKAGE__->has_many( workinghours => "Reports::SelectionWorkinghours" );

=head1 NAME

Reports::Selection - Metadata about a selection of services/hosts/hostgroups

=head1 METHODS

=over 4

=cut

sub set_hostgroups_to {
    my $self = shift;
    Reports::SelectionHostgroup->search( selection => $self->id )->delete_all;
    foreach $_ (@_) {
        $self->add_to_hostgroups( { selection => $self->id, hostgroup => $_ } );
    }
}

=item my_type_is

Returns "report selection"

=cut

sub my_type_is {
    return "report selection";
}

=back

=head1 AUTHOR

Opsera Limited

=head1 LICENSE

GNU General Public License v2

=cut

1;
